package com.adlerosn.brasilfurfest.schedule.abstractDataTypes.managed

import com.adlerosn.brasilfurfest.helper.*
import com.adlerosn.brasilfurfest.schedule.abstractDataTypes.GregorianCalendarRange
import com.adlerosn.brasilfurfest.schedule.abstractDataTypes.convention.Convention
import com.adlerosn.brasilfurfest.schedule.abstractDataTypes.convention.Event
import java.io.Serializable

class AttendeeConFavorite(
        argConvention: Convention,
        argEvent: Event
): UncomplicatedObservable<Any?>(), Serializable {

    private var internalName: String = argEvent.titles.solved?:""
    private var internalRoomName: String = TextualJoiner()[argEvent.places.map { it.names.solved?:"" }]
    private var internalDone: Boolean = false
    private var internalKind: Actions.Intention = if(argEvent.isCustom) Actions.Intention.CUSTOM else Actions.Intention.SELECTION
    private var internalTimeRange: GregorianCalendarRange = argEvent.timeRange
    private var internalEdition: String = argConvention.id
    private var internalConBookId: String = argEvent.conbookId

    var name: String
        get() = internalName
        set(value) = value.let { internalName = it; notifyObservers() }
    var roomName: String
        get() = internalRoomName
        set(value) = value.let { internalRoomName = it; notifyObservers() }
    var kind: Actions.Intention
        get() = internalKind
        set(value) = value.let { internalKind = it; notifyObservers() }
    var timeRange: GregorianCalendarRange
        get() = internalTimeRange
        set(value) = value.let { internalTimeRange = it; notifyObservers() }
    var edition: String
        get() = internalEdition
        set(value) = value.let { internalEdition = it; notifyObservers() }
    var conBookId: String
        get() = internalConBookId
        set(value) = value.let { internalConBookId = it; notifyObservers() }
    var notified: Boolean
        get() = internalDone
        set(value) = value.let { internalDone = it; notifyObservers() }

    fun findInConvention(convention: Convention) = convention.events.firstOrNull { this.relatesTo(it) }
    private val kindStringInt
        get() = Actions.actionString(kind)
    private val doesNameAttributeMatter
        get() = kind in listOf(Actions.Intention.CUSTOM, Actions.Intention.SELECTION, Actions.Intention.ROOMPARTY)
    val isCustom
        get() = conBookId.isEmpty()
    val isSecondaryStringPlace
        get () = kind == Actions.Intention.ROOMPARTY
    val isShareable
        get () = kind in listOf(Actions.Intention.ROOMPARTY)

    fun getString() =
            kindStringInt.let { if(doesNameAttributeMatter) name else it.getString() }
    fun getSecondaryString() =
            Actions.descriptionString(kind)?.getString() ?: roomName
    fun getSecondaryEditString() =
            Actions.descriptionEditString(kind)?.getString() ?: roomName


    override fun equals(other: Any?): Boolean {
        return when {
            other==null -> false
            AttendeeConFavorite::class.isInstance(other) -> {
                val that = other as AttendeeConFavorite
                if(
                        (this.conBookId == that.conBookId)
                        and
                        (this.edition == that.edition)
                        and
                        (this.timeRange == that.timeRange)
                        and
                        (this.kind == that.kind)
                )
                    // if kind is custom, names must be the same to the object be the same
                    ((kind !in listOf(Actions.Intention.CUSTOM, Actions.Intention.ROOMPARTY)) or (this.name == that.name))
                else
                    false
            }
            else -> false
        }
    }

    fun relatesTo(that: Event): Boolean =
            (this.kind == Actions.Intention.SELECTION) and
                    (this.conBookId == that.conbookId) and
                    (this.timeRange == that.timeRange)
}