package com.adlerosn.brasilfurfest.updater.startup

import android.app.Service
import android.content.Intent
import android.os.IBinder
import com.adlerosn.brasilfurfest.helper.lastPathPart
import com.adlerosn.brasilfurfest.helper.nullFun
import com.adlerosn.brasilfurfest.notification.AppJobScheduler
import com.adlerosn.brasilfurfest.schedule.managers.RemoteAssets
import com.adlerosn.brasilfurfest.schedule.managers.ScheduleManagerGetter
import com.adlerosn.brasilfurfest.schedule.managers.UpdateChecker

open class UpdaterService : Service() {
    lateinit var updateChecker: UpdateChecker

    override fun onStartCommand(intent: Intent, flags: Int, startId: Int): Int {
        updateChecker = UpdateChecker(baseContext)
        if(updateChecker.hasBaseJsonCached){
            startSchedulingNotifications()
        }
        updateChecker.downloadBaseJsonWithWatcher { downloadState, _, _ ->
            when(downloadState) {
                UpdateChecker.DownloadState.SUCCESS -> {
                    startSchedulingNotifications()
                    stopSelf()
                }
                UpdateChecker.DownloadState.ERROR -> {
                    if(updateChecker.hasBaseJsonCached)
                        startSchedulingNotifications()
                    stopSelf()
                }
                else -> nullFun()
            }
        }
        return START_STICKY
    }

    val binder = UpdateBinder(this)
    override fun onBind(intent: Intent): IBinder = binder.also { updateChecker = UpdateChecker(baseContext) }
    private fun startSchedulingNotifications() = AppJobScheduler(baseContext)

    fun hasBaseJsonLoaded() = updateChecker.hasBaseJsonCached
    fun updateBaseJson(watcher: (UpdateChecker.DownloadState, ByteArray?, Throwable?)->Any?) =
            updateChecker.downloadBaseJsonWithWatcher(watcher)

    private var sendDownloadSignalsFromMassDownloads = false
    fun downloadImageIfNotCached(path: String, function: () -> Unit) {
        sendDownloadSignalsFromMassDownloads = true
        val url = RemoteAssets.media+path
        if (updateChecker.cacheManager[url.lastPathPart()] == null) {
            updateChecker.downloadWithWatcher(url) { downloadState: UpdateChecker.DownloadState, _: ByteArray?, _: Throwable? ->
                if (sendDownloadSignalsFromMassDownloads)
                    when(downloadState){
                        UpdateChecker.DownloadState.SUCCESS -> function()
                        UpdateChecker.DownloadState.ERROR -> function()
                        else -> nullFun()
                    }
            }
        } else {
            if(sendDownloadSignalsFromMassDownloads) function()
        }
    }

    fun stopAllDownloadSignalsFromMassDownloads(){
        sendDownloadSignalsFromMassDownloads = false
    }
}
